package SeMeet::Schema::Result::User;

use strict;
use warnings;

use base 'DBIx::Class::Core';
use Mojo::JWT;

use constant JWT_DEFAULT_LIFETIME => 3600 * 24 * 7;
use constant AVATAR_SIZE          => 320;
use constant MEET_TOKEN_LIFETIME  => 3600 * 24;

our $VERSION = 1;

__PACKAGE__->table('users');

__PACKAGE__->add_columns(
    id => {
        data_type         => 'integer',
        is_auto_increment => 1,
        is_nullable       => 0,
        sequence          => 'uid_seq'
    },
    qw(
        uuid
        username
        displayname
    ),
);

__PACKAGE__->set_primary_key('id');

__PACKAGE__->add_unique_constraint(
    'uuid' => [qw(uuid)]
);

__PACKAGE__->add_unique_constraint(
    'username' => [qw(username)]
);

__PACKAGE__->has_many(
    groups => 'SeMeet::Schema::Result::Meet',
    { 'foreign.owner_id' => 'self.id', },
);

sub api_token {
    my $self = shift;
    my $args = shift;

    my $exp   = Mojo::JWT->now() + ( $args->{lifetime} // JWT_DEFAULT_LIFETIME );
    my $token = Mojo::JWT->new(
        secret => $args->{secret},
        claims => {
            id          => $self->id,
            groups      => $args->{groups},
            permissions => $args->{permissions},
            exp         => $exp,
        }
    )->encode;

    return $token;
}

sub meet_token {
    my $self = shift;
    my $meet = shift;
    my $cfg  = shift;

    my $avatar = join ('',
       $cfg->{piratar},
       AVATAR_SIZE, '/',
       $self->username,
       '.jpg',
    );

    my $moderator = $meet->meet_users({
        user_id      => $self->id,
        is_moderator => 1,
    })->count;

    return Mojo::JWT->new(
        secret => $cfg->{jitsi_secret},
        claims => {
            aud  => 'semeet',
            iss  => 'semeet',
            sub  => 'meet.pirati.cz',
            room => $meet->uuid,
            moderator => $moderator ? \1:\0,
            exp  => time + MEET_TOKEN_LIFETIME,
            context => {
                user   => {
                    avatar => $avatar,
                    name   => $self->displayname,
#                   email  => $user{mail},
                }
            },
        }
    )->encode;

}

1;
